#!/bin/bash
#
# Copied from pmAIX, modified to support equivalent functions as below:
#
#config_pm.sh -T <ARGS from SMIT>   --> To enable or Disable PM Data transmission
#config_pm.sh -R <ARGS from SMIT>   --> To Set the data retention value in PM_CFGFILE file
#config_pm.sh -S <ARGS from SMIT>   --> To Set the shift timings in PM_CFGFILE File
#
#config_pm.sh -r      --> Retrieve RETAIN value for SMIT display.
#config_pm.sh -s      --> Retrieve Shift timings for SMIT display.
#config_pm.sh status  --> Reports the status of the PM Service. It returns either "Enabled" or "Disabled"
#
Usage()
{
	echo "Usage   : config_pm.sh [options]"
	echo "Function: Set/query PMLinux configuration"
	echo "option -T yes/no   enable/disable PMLinux"
	echo "       -R num      Set to retain <num> days of data files"
	echo "       -S          SetShiftTimings"
	echo "       -r          GetRetainValue"
	echo "       -s          GetShiftTimings"
	echo "       status      isPMEnabled"
}

EnableDisablePM()
{
	while getopts ':T:' flag
	do
		case "$flag" in
			  T) Enable_Pm_Transmission=$OPTARG ;;
			[?]) Usage; exit 10 ;;
		esac
	done
	if [[ ${Enable_Pm_Transmission} = "yes" ]]; then
		#Check for persistent local binary recording
		if [[ ! -f $PM_DAEMON || ! -d "$PM_DATADIR" ]]; then
			echo "Error: PMLinux daemon does NOT exist"
			exit 1
		fi

		#Check if the user has entered Customer information using SMIT
		# Check for email id
		if [[ -z ${Email_Address} ]]; then
			printf "Error: Customer information has not been entered.\n"
			printf "Please enter customer information using cuinfo.PMLnx.\n"
			exit 1
		fi
		# Cron entry for enabling transmission "pmcfg -T"
		# If the entry exists,
		verify.PMLnx -f
		echo "PMLinux Data Transmission enabled!
Note: By enabling PMLinux Service, you agree that IBM may use and share the data
collected by PMLinux for Power Systems within the IBM enterprise without limitation,
including for purposes of problem determination, of assisting you with performance
and capacity planning, of maintaining your existing and new business relationships
with IBM, of notifying you of existing or projected resource constraints, and to
assist us to enhance IBM products. You also agree that your data may be transferred
to such entities in any country whether or not a member of the European Union."
	elif [[ ${Enable_Pm_Transmission} = "no" ]]; then
		# Remove the crontab entry for PM data transmission
		crontab -l | grep "[^#]*${PM_DAEMON}" >/dev/null; rc=$?
		if [[ $rc -eq 0 ]]; then
			uninstall.PMLnx -f; rc1=$?								# with no '-u' : do not remove any directories and files
			rm -f $PM_HOMELN; rc2=$?
			if [[ $rc1 -eq 0 && $rc2 -eq 0 ]]; then
				echo "Successfully disabled PMLinux data transmission"
			else
				echo "Error: Not able to disable PMLinux data transmission".
			fi
		else
			echo "PMLinux Data Transmission is already disabled"
			exit 20
		fi
	else
		Usage
		exit 30
	fi
}

SetRetentionValue()
{
	retain_value=$*
	#Check if the reatin value is "-" or "+". It can happen if a user types '-' or '+' in SMIT
	#without an number
	if [[ -z "$retain_value" ]]; then
		echo "Error: Retention period argument is missing"
		exit 1
	fi
	if [[ $retain_value = "-" || $retain_value = "+" ]]; then
		echo "Error: Invalid retention period - $retain_value."
		echo "Note: Minimum 7 days of retention period is recommended."
		exit 1
	fi

	#Typecast the retain value to integer. 
	#This is used to remove '+', if a user enters values  like '+21'
	typeset -i retain_value=$retain_value

	# Check for values less than or equal to 0
	if [[ $retain_value -lt 1 ]]; then
		echo "Error: Invalid retention period."
		echo "Note: Minimum 7 days of retention period is recommended."
		exit 1
	fi
	tune.PMLnx -f -s PM_RETAIN_DAYS ${retain_value}
	if [[ ${retain_value} -lt 7 ]]; then
		echo "Warning: Minimum 7 days of retention period is recommended."
	fi
}

GetRetainValue()
{
	echo "#RETAIN:"
	echo "${PM_RETAIN_DAYS}:"
}

SetShiftTimings()
{
	Busdays=""
#	Example: config_pm.sh -S -S No -m Yes -t Yes -w Yes -T Yes -f Yes -s No -z 8:00 -Z 15:59 -x 16:00 -X 23:59 -y 0:00 -Y 7:59
	while getopts 'm:t:w:T:f:s:S:z:Z:x:X:y:Y:' option
	do
		case $option in
			m|t|w|T|f|s|S)
			Value=`echo ${OPTARG}| cut -c 1`
			Busdays="${Busdays}${Value}";;

			z)FirstShiftStart=${OPTARG}	;;
			Z)FirstShiftEnd=${OPTARG}	;;
			x)SecShiftStart=${OPTARG}	;;
			X)SecShiftEnd=${OPTARG}		;;
			y)ThirdShiftStart=${OPTARG}
			  setThirdShift=Yes			;;
			Y)ThirdShiftEnd=${OPTARG}
			  setThirdShift=Yes			;;
		esac
	done
	tune.PMLnx -f -s Busdays "${Busdays}"
	tune.PMLnx -f -s Shift_1 "${FirstShiftStart} ${FirstShiftEnd}"
	tune.PMLnx -f -s Shift_2 "${SecShiftStart} ${SecShiftEnd}"
	if [ -n "$setThirdShift" ]; then
	tune.PMLnx -f -s Shift_3 "${ThirdShiftStart} ${ThirdShiftEnd}"
	fi
}

GetShiftTimings()
{
	if [[ -f ${PM_CFGFILE} ]]; then
		first_shift_start=`echo $Shift_1 | awk '{print $1}' | sed 's/:/#!:/g'`
		  first_shift_end=`echo $Shift_1 | awk '{print $2}' | sed 's/:/#!:/g'`
		  sec_shift_start=`echo $Shift_2 | awk '{print $1}' | sed 's/:/#!:/g'`
			sec_shift_end=`echo $Shift_2 | awk '{print $2}' | sed 's/:/#!:/g'`
	fi
	# Collect the Values of Trending Days from ${PM_CFGFILE} file
	shiftSTR="${first_shift_start}:${first_shift_end}:${sec_shift_start}:${sec_shift_end}:"
	for day in 1 2 3 4 5 6 7
	do
		dayStr=
		if [[ -f ${PM_CFGFILE} ]]; then
			dayYN=`echo $Busdays | cut -c $day`
			if [[ ${dayYN} = "Y" ]]; then
				dayStr="Yes,No"
			else
				dayStr="No,Yes"
			fi
		fi
		shiftSTR="$shiftSTR$dayStr:"
	done

	echo "#FirstShiftStart:FirstShiftEnd:SecShiftStart:SecShiftEnd:Sunday:Monday:Tuesday:Wednesday:Thursday:Friday:Saturday:"
	echo $shiftSTR
}

# This function returns the current status of the
# PMLinux data transmission.
# It returns either "Enabled" or "Disabled".
isPMEnabled()
{
	if [[ -L $PM_HOMELN ]]; then
		echo "Enabled"
	else
		echo "Disabled"
	fi
}

setPMEnvs ()
{
	export PM_PRODUCT="PMLinux"
	export PM_EXT="PMLnx"
	export PM_VERSION="1.0-1.1"
	export PM_HOME=${PM_HOME:-/var/perf/pm}
	export PM_BINDIR=$PM_HOME/bin
	export PM_TMPDIR=$PM_HOME/tmp
	export PM_CFGDIR=$PM_HOME/config
	export PM_DATADIR=$PM_HOME/daily/`uname -n`
	export PM_MANDIR=/usr/share/man/man8

	export PM_DAEMON=$PM_BINDIR/daemon_${PM_EXT}
	export PM_LOGFILE=$PM_HOME/daily/pmcfg.log					# was $PM_PRODUCT.log
	export PM_CFGFILE=$PM_CFGDIR/$PM_PRODUCT.cfg

	export PM_CMDFILE=`readlink -f $PROGNAME`
	export PROGNAME=`basename $PROGNAME`
	export PM_CMDPATH=`dirname $PM_CMDFILE`

	export PATH=$PATH:$PM_BINDIR
	export PM_HOMELN=/var/adm/perfmgr
	if [ -s "$PM_CFGFILE" ]; then
		sed "/PM_DATADIR=/d;/PM_HOME=/d" $PM_CFGFILE > /tmp/CFG.tmp_$PROGNAME
		.  /tmp/CFG.tmp_$PROGNAME								# using values in .cfg as defaults
		rm /tmp/CFG.tmp_$PROGNAME
	fi
}

main()
{
	Is_Linux=`uname -s`
	if [[ ${Is_Linux} != "Linux" ]]; then
		echo "PMLinux Service is not supported on non-Linux System"
		exit 1
	fi

	setPMEnvs
	echo "`date '+%Y-%m-%d %T'` $PPID(`readlink -f /proc/$PPID/exe` calling)$0 $*" >>${PM_LOGFILE}

	ARG1=$1
	if   [[ ${ARG1} = "-T" ]]; then
		EnableDisablePM $*
	elif [[ ${ARG1} = "-R" ]]; then
		shift
		SetRetentionValue $*
	elif [[ ${ARG1} = "-S" ]]; then
		shift
		SetShiftTimings $*
	elif [[ ${ARG1} = "-r" ]]; then
		shift
		GetRetainValue
	elif [[ ${ARG1} = "-s" ]]; then
		shift
		GetShiftTimings
	elif [[ ${ARG1} = "-h" ]]; then
		Usage
	elif [[ ${ARG1} = "status" ]]; then						# Reports the status of the PM. It returns either "Enabled" or "Disabled"
		shift
		isPMEnabled
	fi
}

PROGNAME=$0
main $*
exit 0

